;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2009
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
;
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
; ccurve_test
;
; PURPOSE:
;	Tests a TES ozone profile for a "C-curve" condition (as discussed
;	in the TES Level 2 Data Users Guide for V004 data,
;	section 5.1.1.2) and returns a 0 (zero) if a c-curve is found.
;	C-curces are anomalous ozone retrievals.  This is a supplemental
;	test to the TES master quality flag to screen out more suspicious
;	cases.  Not all cases flaged by this test are automatically bad,
;	but should be treated with caution and other information examined
;	before use.  Only for ozone products.
;
; CALL:
; 	c_curve_flag = ccurve_test(TES_pressure, O3_initial,
; 		       O3_retrieved)
;
; INPUTS:
;	TES_pressure = TES pressure array  (species_data.pressure)
;	O3_initial  = TES O3 Initial guess profile (species_data.initial)
;	O3_retrieved = TES O3 retrieved profile (species_data.species)
;
;		(names in parenthesis are names from the tes_l2_hdf
;		product file reader data structure)
;
; OUTPUT:  c_curve_flag = 1 for good data (no c-curve found)
;			  0 for probable bad data (c-curve condition found)
;
; ORIGINAL AUTHOR:
;	Brendan Fisher, JPL, Jan. 2009.  Streamlined version of function
;	provided by H. Worden.
;
; UPDATE HISTORY:
;
;-


FUNCTION Ccurve_test, TES_pressure, O3_initial, O3_retrieved

;;TES_pressure = array of pressures for retrieved profile 
;;O3_initial = TES O3 initial guess 
;;O3_retrieved = TES O3 retrieved 

ccflag = 1 ;; default to good quality 

inlo = where(TES_pressure ge 700.,lo_count)
inhi = where((TES_pressure ge 200.) and (TES_pressure le 350.),hi_count)   

if lo_count gt 0 and hi_count gt 0 then begin
    avg_ig = mean(o3_initial[inlo])
    avg_lo = mean(o3_retrieved[inlo])
    avg_hi = mean(o3_retrieved[inhi])
    if ((avg_lo/avg_ig gt 1.6) and $
        (avg_lo/avg_hi gt 1.4)) then ccflag = 0   

endif 



return, ccflag 

END 
