;+
;
;                      Tropospheric Emission
;                          Spectrometer
;                             (TES)
;
;                         Copyright 2009
;              California Institute of Technology.
;                       ALL RIGHTS RESERVED.
;           U.S. Government Sponsorship acknowledged.
;
;           Any commercial use must be negotiated with the
;           Office of Technology Transfer at the California
;           Institute of Technology.
;
; 
;  This software may be subject to U.S. export control laws. 
;  By accepting this software, the user agrees to comply with all 
;  applicable U.S. export laws and regulations. User has the 
;  responsibility to obtain export licenses, or other export 
;  authority as may be required before exporting such information 
;  to foreign countries or providing access to foreign persons.
;
; get_tes_outlier
;
; PURPOSE:
;	Tests a set of TES ozone profiles for anomalous 'c-curve'
;	conditions and returns an array of flags, 0 for bad profiles,
;	1 for good profiles.  This quality flag is supplemental to the
;	TES master quality flag to screen out more suspicious cases.
;	Not all cases flaged by this test are automatically bad, but 
;	should be treated with caution and other information examined 
;	before use.  Only for ozone products.
;
; CALL:
;	flags = get_tes_outlier(TES_pressure,TES_ozone,
;		TES_initial,TES_AK_diagonal)
;
; INPUT:
;	TES_pressure = TES pressure array (67xN array, N=# of scenes)
;	TES_ozone = TES retrieved ozone (67xN array)
;	TES_intial = TES ozone initial (67xN array)
; 	TES_AK_diagonal = TES averaging kernel diagonal (67xN array)
;
; OUTPUT:
; 	flags = array of N elements
;		1 for good data 
;		0 flagged as bad data
;
; ORIGINAL AUTHOR:
;	Lin Zhang, Harvard University
;
; UPDATE HISTORY:
;	Brendan Fisher, JPL, Feb. 2009.  Minor updates & optimizations,
;	header documentation.  Array orientation swapped to match reader
;	programs.
;
;-
;	
; edit by Lin Zhang 09/03/2008
; updated 10/18/2008
; this function is for finding abnormally high TES ozone values in the
; lower troposphere (like C-curve)
;
;=====================================================================
; To test the C-curve observations:
; Let maxlo be the largest retrieval value for levels > 700 hPa 
; and minhi be the smallest retrieval value for levels at 700-200hPa
; and surf  be the retrieved surface concentration
; and maxratio be the maximum of the (retrieval/a priori) ratio for
;                 levels > 700 hPa 
; and avkern be the mean value of the diagonal terms of the averaging
;               kernel for pressure levels > 700 hPa
; The ozone retrieval can be considered as a C-curve profile if any of
; the following conditions occurs:
; 1) maxlo > 150 ppbv 
; 2) (maxlo > 100 ppbv or maxratio > 1.8) and avkern < 0.1
; 3) maxlo / minhi > 2.5 or 
;    (maxlo / minhi > 2.0 and maxlo / surf > 1.1)
;======================================================================


function get_tes_outlier, p_tes, o3_tes, o3_tes_ig, ak_diag

; dimension of p_tes [number of pressure levels,number of obs]
result = Size( p_tes, /Dim )
nn = result[1]

; define output array
cflag = intarr(nn)  &  cflag[*] = 1

; loop for each observation
for i = 0L, nn-1 do begin

  ; default is good = 1
    test1 = 1
    test2 = 1
    test3 = 1

    p_tmp = reform( p_tes[*,i] )
    o3_tmp = reform( o3_tes[*,i] )
    o3_ig_tmp = reform( o3_tes_ig[*,i] )
    ak_tmp = reform( ak_diag[*,i] )

    ind = where( p_tmp gt 700, num )
    if num gt 0 then begin
        
      ;------------------------------
      ; test 1 
      ; max > 150 ppbv for pressure levels greater than 700 hPa
      ;------------------------------
        o3_test = where( o3_tmp[ind] gt 1.5d-7 )
        if o3_test[0] ne -1 then test1 = 0 else test1 = 1

      ;-------------------------------
      ; test 2
      ; for levels greater than 700 hPa
      ; (> 100 ppbv below 700 hPa or retrieval/a priori > 1.8)
      ; and (averaged diagonal terms of AK for these level < 0.1)
      ; NOTE:  this is to remove some profiles that have
      ; large values near surface but no enough sensitivity
      ;--------------------------------
        o3_test = where( o3_tmp[ind] gt 1.0d-7 or o3_tmp[ind]/o3_ig_tmp[ind] ge 1.8 )
;        ak_test = total(ak_tmp[ind])/double(num)
        ak_test = mean(ak_tmp[ind])
        if (o3_test[0] ne -1 and ak_test le 0.1) then $
           test2 = 0 else test2 = 1
    endif

      ;---------------------------------------------------
      ; test 3
      ; C-Shaped
      ; let maxlo be the largest value for levels greater than 700 hPa
      ; and minhi be the smallest value for levels from 700 hPa to 200 hPa
      ; and surf be the surface concentration
      ; the C-curve is considered if :
      ; 1) maxlo/minhi > 2.5
      ; or 2) maxlo/minhi > 2 and maxlo/surf > 1.1 (use of 1.05 - 1.1
      ;       doesn't change the comparison with available sondes)
      ;---------------------------------------------------
    inlo = where( p_tmp ge 700.,lo_count)
    inhi = where((p_tmp ge 200.) and (p_tmp le 700.), hi_count)

    if lo_count gt 0 and hi_count gt 0 then begin
        surf = o3_tmp[inlo[0]]  ; Surface value
        maxlo = max( o3_tmp(inlo) )
        minhi = min( o3_tmp(inhi) )

        if ( maxlo/minhi gt 2.5 ) then test3 = 0 else test3 = 1
        if ( maxlo/minhi ge 2. and maxlo/surf ge 1.1 ) then test3 = 0 

    endif 

    if ( test1 eq 0 or test2 eq 0 or test3 eq 0 ) then $
      cflag[i] = 0

endfor

; output cflag[number of obs]
return, cflag

end






